/******************************************************************
 * Copyright (C) 2005 by SG Software.
 *
 * SG MPFC. LIRC support plugin.
 * $Id$ 
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public 
 * License along with this program; if not, write to the Free 
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, 
 * MA 02111-1307, USA.
 */

//#include "config.h"
#define __MPFC_OUTER__
#include <lirc/lirc_client.h>
#include <sys/select.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <mpfc/util.h>
#include "types.h"
#include "genp.h"
#include "pmng.h"
#include "wnd_editbox.h"
#include "wnd_repval.h"

/* Plugins manager */
static pmng_t *lirc_pmng = NULL;

/* Plugin description */
static char *lirc_desc = "LIRC support plugin.";

/* Plugin author */
static char *lirc_author = "Sergey E. Galanov <sgsoftware@mail.ru>";

/* Logger */
static logger_t *lirc_log = NULL;

/* Whether plugin is started */
static bool_t lirc_started = FALSE;

/* LIRC configuration */
static struct lirc_config *lirc_cfg_obj = NULL;

/* LIRC communication socket */
static int lirc_sock = -1;

/* Thread id */
static pthread_t lirc_tid = 0;
static bool_t lirc_end_thread = FALSE;

/* MPFC configuration */
static cfg_node_t *lirc_cfg = NULL;

/* Repeat value input dialog */
static dialog_t *lirc_repval_dlg = NULL;

/* Handle 'on_ok' message for repeat value dialog */
static wnd_msg_retcode_t lirc_repval_on_ok( wnd_t *wnd )
{
	return WND_MSG_RETCODE_OK;
} /* End of 'lirc_repval_on_ok' function */

/* Handle 'close' message for repeat value dialog */
static wnd_msg_retcode_t lirc_repval_on_close( wnd_t *wnd )
{
	lirc_repval_dlg = NULL;
	return WND_MSG_RETCODE_OK;
} /* End of 'lirc_repval_on_ok' function */

/* Input a digit into repeat value dialog */
static void lirc_digit2repval( char d )
{
	/* Create dialog if it is not created */
	if (lirc_repval_dlg == NULL)
	{
		lirc_repval_dlg = wnd_repval_new(lirc_pmng->m_root_wnd, 
				lirc_repval_on_ok, d - '0');
		wnd_msg_add_handler(WND_OBJ(lirc_repval_dlg), "close",
				lirc_repval_on_close);
	}
	/* Redirect digit to the dialog */
	else 
	{
		wnd_t *count = WND_OBJ(dialog_find_item(lirc_repval_dlg, "count"));
		if (count == NULL)
			return;

		wnd_msg_send(count, "keydown", wnd_msg_keydown_new(d));
	}
} /* End of 'lirc_digit2repval' function */

/* Events capturing thread */
static void *lirc_thread( void *arg )
{
	logger_debug(lirc_log, "lirc: in lirc_thread");

	for ( ; !lirc_end_thread; )
	{
		char *code, *c;
		int ret;
		fd_set fds;
		struct timeval tv;

		/* Check for input */
		FD_ZERO(&fds);
		FD_SET(lirc_sock, &fds);
		memset(&tv, 0, sizeof(tv));
		ret = select(lirc_sock + 1, &fds, NULL, NULL, &tv);
		if (ret <= 0)
		{
			util_wait();
			continue;
		}

		/* Read code */
		if (lirc_nextcode(&code) != 0)
			break;
		if (code == NULL)
			continue;

		/* Translate into commands */
		logger_debug(lirc_log, "lirc: got a code");
		while (((ret = lirc_code2char(lirc_cfg_obj, code, &c)) == 0) &&
					c != NULL)
		{
			int repval = 0;

			logger_debug(lirc_log, "lirc: got command %s", c);

			/* Handle digits in a different way */
			if (isdigit(*c) && !(*(c + 1)))
			{
				lirc_digit2repval(*c);
				continue;
			}
			/* Press 'cancel' */
			else if (lirc_repval_dlg != NULL && !strcmp(c, "repval_cancel"))
			{
				wnd_msg_send(WND_OBJ(lirc_repval_dlg), "cancel_clicked",
						dialog_msg_cancel_new());
				continue;
			}

			/* Get repeat value */
			if (lirc_repval_dlg != NULL)
			{
				repval = atoi(EDITBOX_TEXT(dialog_find_item(lirc_repval_dlg, 
								"count")));
				logger_debug(lirc_log, "lirc: repeat value is %d", repval);
				wnd_msg_send(WND_OBJ(lirc_repval_dlg), "ok_clicked",
						dialog_msg_ok_new());
			}

			/* Send action */
			pmng_player_command(lirc_pmng, "action", "si", c, repval);
		}
		free(code);

		if (ret == -1)
			break;

	}
	logger_debug(lirc_log, "lirc: returned from thread cycle");
	return NULL;
} /* End of 'lirc_thread' function */

/* Start plugin */
bool_t lirc_start( void )
{
	int ret;

	logger_debug(lirc_log, "lirc: in lirc_start");

	/* Initialize lirc */
	lirc_sock = lirc_init("mpfc", 0);
	if (lirc_sock < 0)
	{
		logger_error(lirc_log, 0, "lirc: lirc_init failed");
		return FALSE;
	}

	/* Read configuration */
	ret = lirc_readconfig(cfg_get_var(lirc_cfg, "cfg"), &lirc_cfg_obj, NULL);
	if (ret != 0)
	{
		logger_error(lirc_log, 0, "lirc: lirc_readconfig failed");
		lirc_deinit();
		return FALSE;
	}

	/* Start events capturing thread */
	ret = pthread_create(&lirc_tid, NULL, lirc_thread, NULL);
	if (ret < 0)
	{
		logger_error(lirc_log, 0, "lirc: pthread_create failed");
		lirc_freeconfig(lirc_cfg_obj);
		lirc_cfg_obj = NULL;
		lirc_deinit();
		return FALSE;
	}
	lirc_started = TRUE;
	return TRUE;
} /* End of 'lirc_start' function */

/* End plugin */
void lirc_end( void )
{
	lirc_started = FALSE;

	if (lirc_tid != 0)
	{
		lirc_end_thread = TRUE;
		pthread_join(lirc_tid, NULL);
		lirc_end_thread = FALSE;
		lirc_tid = 0;
	}

	if (lirc_cfg_obj != NULL)
	{
		lirc_freeconfig(lirc_cfg_obj);
		lirc_cfg_obj = NULL;
	}
	if (lirc_sock >= 0)
	{
		lirc_deinit();
		lirc_sock = -1;
	}
} /* End of 'lirc_end' function */

/* Check whether plugin is started */
bool_t lirc_is_started( void )
{
	return lirc_started;
} /* End of 'lirc_is_started' function */

/* Exchange data with main program */
void plugin_exchange_data( plugin_data_t *pd )
{
	pd->m_desc = lirc_desc;
	pd->m_author = lirc_author;
	GENP_DATA(pd)->m_start = lirc_start;
	GENP_DATA(pd)->m_is_started = lirc_is_started;
	GENP_DATA(pd)->m_end = lirc_end;
	lirc_pmng = pd->m_pmng;
	lirc_log = pd->m_logger;
	lirc_cfg = pd->m_cfg;
} /* End of 'plugin_exchange_data' function */

/* End of 'lirc.c' file */

